@@ -98,7 +98,8 @@ class SameTypeVisitor(TypeVisitor[bool]):
 
     def visit_tuple_type(self, left: TupleType) -> bool:
         if isinstance(self.right, TupleType):
-            return is_same_types(left.items, self.right.items)
+            return (is_same_type(left.fallback, self.right.fallback)
+                    and is_same_types(left.items, self.right.items))
         else:
             return False
 
