@@ -15,7 +15,7 @@ from mypy.options import Options
 from mypy.server.deps import get_dependencies
 from mypy.test.config import test_temp_dir
 from mypy.test.data import DataDrivenTestCase, DataSuite
-from mypy.test.helpers import assert_string_arrays_equal
+from mypy.test.helpers import assert_string_arrays_equal, parse_options
 from mypy.types import Type
 from mypy.typestate import TypeState
 
@@ -41,7 +41,13 @@ class GetDependenciesSuite(DataSuite):
             python_version = defaults.PYTHON2_VERSION
         else:
             python_version = defaults.PYTHON3_VERSION
-        messages, files, type_map = self.build(src, python_version)
+        options = parse_options(src, testcase, incremental_step=1)
+        options.use_builtins_fixtures = True
+        options.show_traceback = True
+        options.cache_dir = os.devnull
+        options.python_version = python_version
+        options.export_types = True
+        messages, files, type_map = self.build(src, options)
         a = messages
         if files is None or type_map is None:
             if not a:
@@ -53,7 +59,7 @@ class GetDependenciesSuite(DataSuite):
                                                                            'typing',
                                                                            'mypy_extensions',
                                                                            'enum'):
-                    new_deps = get_dependencies(files[module], type_map, python_version)
+                    new_deps = get_dependencies(files[module], type_map, python_version, options)
                     for source in new_deps:
                         deps[source].update(new_deps[source])
 
@@ -75,15 +81,9 @@ class GetDependenciesSuite(DataSuite):
 
     def build(self,
               source: str,
-              python_version: Tuple[int, int]) -> Tuple[List[str],
-                                                        Optional[Dict[str, MypyFile]],
-                                                        Optional[Dict[Expression, Type]]]:
-        options = Options()
-        options.use_builtins_fixtures = True
-        options.show_traceback = True
-        options.cache_dir = os.devnull
-        options.python_version = python_version
-        options.export_types = True
+              options: Options) -> Tuple[List[str],
+                                         Optional[Dict[str, MypyFile]],
+                                         Optional[Dict[Expression, Type]]]:
         try:
             result = build.build(sources=[BuildSource('main', None, source)],
                                  options=options,
