@@ -61,7 +61,6 @@ class TransformVisitor(NodeVisitor[Node]):
         # NOTE: The 'names' and 'imports' instance variables will be empty!
         new = MypyFile(self.statements(node.defs), [], node.is_bom,
                        ignored_lines=set(node.ignored_lines))
-        new._name = node._name
         new._fullname = node._fullname
         new.path = node.path
         new.names = SymbolTable()
@@ -94,7 +93,7 @@ class TransformVisitor(NodeVisitor[Node]):
 
         # These contortions are needed to handle the case of recursive
         # references inside the function being transformed.
-        # Set up placholder nodes for references within this function
+        # Set up placeholder nodes for references within this function
         # to other functions defined inside it.
         # Don't create an entry for this function itself though,
         # since we want self-references to point to the original
@@ -106,7 +105,7 @@ class TransformVisitor(NodeVisitor[Node]):
         new = FuncDef(node.name(),
                       [self.copy_argument(arg) for arg in node.arguments],
                       self.block(node.body),
-                      cast(FunctionLike, self.optional_type(node.type)))
+                      cast(Optional[FunctionLike], self.optional_type(node.type)))
 
         self.copy_function_attributes(new, node)
 
@@ -132,8 +131,8 @@ class TransformVisitor(NodeVisitor[Node]):
 
     def visit_lambda_expr(self, node: LambdaExpr) -> LambdaExpr:
         new = LambdaExpr([self.copy_argument(arg) for arg in node.arguments],
-                       self.block(node.body),
-                       cast(FunctionLike, self.optional_type(node.type)))
+                         self.block(node.body),
+                         cast(Optional[FunctionLike], self.optional_type(node.type)))
         self.copy_function_attributes(new, node)
         return new
 
@@ -154,6 +153,9 @@ class TransformVisitor(NodeVisitor[Node]):
         new._fullname = node._fullname
         new.type = self.optional_type(node.type)
         new.info = node.info
+        new.is_static = node.is_static
+        new.is_class = node.is_class
+        new.is_property = node.is_property
         if node.impl:
             new.impl = cast(OverloadPart, node.impl.accept(self))
         return new
@@ -401,7 +403,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return ListExpr(self.expressions(node.items))
 
     def visit_dict_expr(self, node: DictExpr) -> DictExpr:
-        return DictExpr([(self.expr(key), self.expr(value))
+        return DictExpr([(self.expr(key) if key else None, self.expr(value))
                          for key, value in node.items])
 
     def visit_tuple_expr(self, node: TupleExpr) -> TupleExpr:
@@ -475,8 +477,7 @@ class TransformVisitor(NodeVisitor[Node]):
                            self.type(node.upper_bound), variance=node.variance)
 
     def visit_type_alias_expr(self, node: TypeAliasExpr) -> TypeAliasExpr:
-        return TypeAliasExpr(node.type, node.tvars,
-                             fallback=node.fallback, in_runtime=node.in_runtime)
+        return TypeAliasExpr(node.type, node.tvars, node.no_args)
 
     def visit_newtype_expr(self, node: NewTypeExpr) -> NewTypeExpr:
         res = NewTypeExpr(node.name, node.old_type, line=node.line)
@@ -584,7 +585,7 @@ class TransformVisitor(NodeVisitor[Node]):
 class FuncMapInitializer(TraverserVisitor):
     """This traverser creates mappings from nested FuncDefs to placeholder FuncDefs.
 
-    The placholders will later be replaced with transformed nodes.
+    The placeholders will later be replaced with transformed nodes.
     """
 
     def __init__(self, transformer: TransformVisitor) -> None:
