@@ -413,7 +413,10 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
                 variables = self.bind_function_type_variables(t, t)
             ret = t.copy_modified(arg_types=self.anal_array(t.arg_types, nested=nested),
                                   ret_type=self.anal_type(t.ret_type, nested=nested),
-                                  fallback=t.fallback or self.named_type('builtins.function'),
+                                  # If the fallback isn't filled in yet,
+                                  # its type will be the falsey FakeInfo
+                                  fallback=(t.fallback if t.fallback.type
+                                            else self.named_type('builtins.function')),
                                   variables=self.anal_var_defs(variables))
         return ret
 
@@ -437,7 +440,9 @@ class TypeAnalyser(SyntheticTypeVisitor[Type], TypeAnalyzerPluginInterface):
             else:
                 return AnyType(TypeOfAny.from_error)
         any_type = AnyType(TypeOfAny.special_form)
-        fallback = t.fallback if t.fallback else self.named_type('builtins.tuple', [any_type])
+        # If the fallback isn't filled in yet, its type will be the falsey FakeInfo
+        fallback = (t.fallback if t.fallback.type
+                    else self.named_type('builtins.tuple', [any_type]))
         return TupleType(self.anal_array(t.items), fallback, t.line)
 
     def visit_typeddict_type(self, t: TypedDictType) -> Type:
@@ -708,28 +713,7 @@ class TypeAnalyserPass3(TypeVisitor[None]):
             self.indicator['synthetic'] = True
         # Check type argument count.
         if len(t.args) != len(info.type_vars):
-            if len(t.args) == 0:
-                any_type = AnyType(TypeOfAny.from_omitted_generics,
-                                   line=t.line, column=t.column)
-                t.args = [any_type] * len(info.type_vars)
-                return
-            # Invalid number of type parameters.
-            n = len(info.type_vars)
-            s = '{} type arguments'.format(n)
-            if n == 0:
-                s = 'no type arguments'
-            elif n == 1:
-                s = '1 type argument'
-            act = str(len(t.args))
-            if act == '0':
-                act = 'none'
-            self.fail('"{}" expects {}, but {} given'.format(
-                info.name(), s, act), t)
-            # Construct the correct number of type arguments, as
-            # otherwise the type checker may crash as it expects
-            # things to be right.
-            t.args = [AnyType(TypeOfAny.from_error) for _ in info.type_vars]
-            t.invalid = True
+            fix_instance(t, self.fail)
         elif info.defn.type_vars:
             # Check type argument values. This is postponed to the end of semantic analysis
             # since we need full MROs and resolved forward references.
@@ -808,6 +792,8 @@ class TypeAnalyserPass3(TypeVisitor[None]):
         if t.resolved is None:
             resolved = self.anal_type(t.unbound)
             t.resolve(resolved)
+            assert t.resolved is not None
+            t.resolved.accept(self)
 
     def anal_type(self, tp: UnboundType) -> Type:
         tpan = TypeAnalyser(self.api,
@@ -824,6 +810,35 @@ class TypeAnalyserPass3(TypeVisitor[None]):
 TypeVarList = List[Tuple[str, TypeVarExpr]]
 
 
+def fix_instance(t: Instance, fail: Callable[[str, Context], None]) -> None:
+    """Fix a malformed instance by replacing all type arguments with Any.
+
+    Also emit a suitable error if this is not due to implicit Any's.
+    """
+    if len(t.args) == 0:
+        any_type = AnyType(TypeOfAny.from_omitted_generics,
+                           line=t.line, column=t.column)
+        t.args = [any_type] * len(t.type.type_vars)
+        return
+    # Invalid number of type parameters.
+    n = len(t.type.type_vars)
+    s = '{} type arguments'.format(n)
+    if n == 0:
+        s = 'no type arguments'
+    elif n == 1:
+        s = '1 type argument'
+    act = str(len(t.args))
+    if act == '0':
+        act = 'none'
+    fail('"{}" expects {}, but {} given'.format(
+        t.type.name(), s, act), t)
+    # Construct the correct number of type arguments, as
+    # otherwise the type checker may crash as it expects
+    # things to be right.
+    t.args = [AnyType(TypeOfAny.from_error) for _ in t.type.type_vars]
+    t.invalid = True
+
+
 def expand_type_alias(target: Type, alias_tvars: List[str], args: List[Type],
                       fail: Callable[[str, Context], None], no_args: bool, ctx: Context) -> Type:
     """Expand a (generic) type alias target following the rules outlined in TypeAlias docstring.
@@ -857,7 +872,12 @@ def expand_type_alias(target: Type, alias_tvars: List[str], args: List[Type],
              % (exp_len, act_len), ctx)
         return set_any_tvars(target, alias_tvars or [],
                              ctx.line, ctx.column, implicit=False)
-    return replace_alias_tvars(target, alias_tvars, args, ctx.line, ctx.column)
+    typ = replace_alias_tvars(target, alias_tvars, args, ctx.line, ctx.column)
+    # HACK: Implement FlexibleAlias[T, typ] by expanding it to typ here.
+    if (isinstance(typ, Instance)
+            and typ.type.fullname() == 'mypy_extensions.FlexibleAlias'):
+        typ = typ.args[-1]
+    return typ
 
 
 def replace_alias_tvars(tp: Type, vars: List[str], subs: List[Type],
